<?php

class FileManager
{
	private $requestDir;
	private $filesDir;
	private $filesUrl;
	private $currDir;
	private $currUrl;
	private $managerUrl;
	private $maxFileSize = 5000000; // in bytes
	
	public function __construct($options)
	{
		$this->requestDir = $options['requestDir'];
		$this->filesDir = $options['filesDir'];
		$this->filesUrl = $options['filesUrl'];
		$this->managerUrl = $options['managerUrl'];
		
		// Validate and set request directory paths
		$reqDir = $this->validate($this->filesDir.$this->requestDir);
		if (!$reqDir)
		{
			$this->currDir = $this->filesDir;
			$this->currUrl = $this->filesUrl;
			$this->requestDir = '';
		}
		else
		{
			if (substr($this->requestDir, 0, -1) != '/') $this->requestDir .= '/';
			if (substr($this->requestDir, 0, 1) == '/') $this->requestDir = substr($this->requestDir, 1);
			$this->currDir = $this->filesDir.$this->requestDir;
			$this->currUrl = $this->filesUrl.$this->requestDir;
		}
	}
	
	/*
	 * Ensure that the file or folder is located within viewable directory and is readable
	 */
	function validate($path)
	{
		$path = str_replace('\\', '/', realpath($path)).'/';
		if (!preg_match('%^'.preg_quote($this->filesDir, '%').'%i', $path) || !is_readable($path))
			return false;
		return $path;
	}
	
	/*
	 * Strip dangerous symbols from the directory path
	 */
	function sanitize($name)
	{
		$name = str_replace(array('..', '.\\', '\.', './', '/.'), '', $name);
		return $name;
	}
	
	/*
	 * Pick an icon to represent the filetype
	 */
	function getFiletype($filename)
	{
		$ext = file_ext($filename);
		
		switch ($ext)
		{
			case 'jpg':
			case 'jpeg':
			case 'png':
			case 'gif':
			case 'tiff':
				return 'image';
			case 'pdf':
				return 'pdf';
			case 'doc':
			case 'docx':
			case 'txt':
				return 'document';
			case 'zip':
				return 'zip';
			case 'avi':
			case 'wmv':
			case 'mov':
				return 'video';
			default:
				return 'dir';
		}
	}
	
	/*
	 * Checks if prev directory exists
	 */
	function dirGetPrev()
	{
		$files_dir = preg_quote($this->filesDir, '/');
		$prev_dir = $this->validate($this->currDir.'../');
		if ($prev_dir)
			return preg_replace('/^'.$files_dir.'/ui', '', $prev_dir);
		return false;
	}
	
	/*
	 * Get files and directories within a selected directory in HTML format
	 */
	function dirView()
	{
		$prevDir = $this->dirGetPrev();
		if ($prevDir !== false)
		{
			$prevDir = str_replace('\\', '\\\\', $prevDir);
			$htmlDirs = <<<EOT
<tr>
	<td class="icon"><img src="images/dir.png" alt="Directory" /></td>
	<td colspan="4">
		<a href="#" onclick="DmFileManager.changeDir('$prevDir'); return false;">Previous directory</a>
	</td>
</tr>
EOT;
		}
		else
		{
			$htmlDirs = '';
		}
		$htmlFiles = '';
		
		$dP = opendir($this->currDir);
		if ($dP)
		{
			$fileId = 0;
			while ($file = readdir($dP))
			{
				if ($file == '.' || $file == '..') continue;
				
				$is_dir = is_dir($this->currDir.$file);
				
				$fileurl = $this->currUrl.$file;
				$selectLink = '';
				
				if ($is_dir)
				{
					$fileIdText = '';
					$filename = '<a href="#" onclick="DmFileManager.changeDir(\''.$this->requestDir.$file.'\'); return false;">'.$file.'</a>';
					$filesize = dir_size($this->currDir.$file);
					$deleteJsFunc = 'DmFileManager.dirDelete(\''.urlencode($file).'\'); return false;';
					$renameLink = '<li><a href="#" onclick="DmFileManager.rename(\'dir\', \''.$file.'\'); return false;">Rename</a></li>';
					$downloadLink = '';
				}
				else
				{
					$fileIdText = 'file'.$fileId++;
					$filename = $file;
					$filesize = filesize($this->currDir.$file);
					$deleteJsFunc = 'DmFileManager.fileDelete(\''.urlencode($file).'\'); return false;';
					$selectLink = '<li><a href="#" onclick="DmFileManager.fileInsert(\''.$fileIdText.'\'); return false;">Select</a></li>';
					$renameLink = '<li><a href="#" onclick="DmFileManager.rename(\'file\', \''.$file.'\'); return false;">Rename</a></li>';
					$downloadLink = '<li><a href="'.$fileurl.'">Download</a></li>';
				}
				
				$filesize = ceil($filesize / 1024);
				$filetype = $this->getFiletype($file);
				
				if ($filetype == 'image')
					$filePreview = '<a href="#" onclick="window.open(\''.$fileurl.'\', \'ImagePreview\', \'toolbar=no\'); return false;"><img src="'.$this->managerUrl.'images/'.$filetype.'.png" alt="not found" /></a>';
				else
					$filePreview = '<img src="'.$this->managerUrl.'images/'.$filetype.'.png" alt="not found" />';
				
        $deleteLink = '<li><a href="#" onclick="'.$deleteJsFunc.'">Delete</a></li>';  
        
				$html = <<<EOT
<tr id="$fileIdText">
	<td class="icon">$filePreview</td>
	<td>$filename</td>
	<td>$fileurl</td>
	<td>$filesize KB</td>
	<td>
		<div class="options" onmouseout="dmOptionsHide(this)" onmouseover="dmOptionsShow(this)">
			<a href="#">Options</a>
			<ul style="display:none;">
				$selectLink
				$deleteLink
				$renameLink
				$downloadLink
			</ul>
		</div>
	</td>
</tr>
EOT;
				if ($is_dir) $htmlDirs .= $html;
				else $htmlFiles .= $html;
			}
			closedir($dP);
		}
		return '<tr><th width="1%"></th><th width="20%">Filename</th><th width="45%">Url</th><th width="15%">Size</th><th width="19%"></th></tr>'.$htmlDirs.$htmlFiles;
	}
	
	public function fileUpload()
	{
		if (isset($_FILES['file']['tmp_name']))
		{
			$filename = sanitize_filename($_FILES['file']['name']);
			if ($filename == '') return 'error_filename';
			$filetype = file_type($filename);
			$filesize = filesize($_FILES['file']['tmp_name']);
			
			if ($filesize >= $this->maxFileSize)
				return 'error_filesize';
			
			if ($filetype != 'unknown')
			{
				$path = $this->currDir.$filename;
				
				// Check if file exists
				if (file_exists($path))
					return 'error_fileexists';
				
				// Move
				if (move_uploaded_file($_FILES['file']['tmp_name'], $path))
					return 'success';
				else
					return 'failure';
			}
			else
				return 'error_filetype';
		}
		return 'error_fileempty';
	}
	
	public function fileDelete()
	{
		$filename = isset($_GET['filename']) ? $_GET['filename'] : '';
		
		if ($filename != '')
		{
			$path = $this->sanitize($this->currDir.$filename);
			if (file_exists($path))
			{
				if (unlink($path))
					return 1;
			}
		}
		return 0;
	}
	
	public function dirCreate($mod = 0644)
	{
		if (!isset($_POST['dirname']) || empty($_POST['dirname']))
			return 0;
		else
			$dirname = sanitize_filename($_POST['dirname'], true);
		
		if ($dirname == '') return 0;
		
		$newdir = $this->currDir.$dirname;
		
		if (!file_exists($newdir))
			return mkdir($newdir, $mod);
		return 0;
	}
	
	public function dirDelete()
	{
		$dirname = isset($_GET['dirname']) ? $_GET['dirname'] : '';
		if ($dirname != '')
		{
			$path = $this->sanitize($this->currDir.$dirname);
			if (file_exists($path))
			{
				if (rmdir($path))
					return 1;
			}
		}
		return 0;
	}
	
	public function rename()
	{
		$oldname = isset($_POST['oldname']) ? sanitize_filename($_POST['oldname']) : '';
		$newname = isset($_POST['newname']) ? sanitize_filename($_POST['newname']) : '';
		
		if ($oldname == '' || $newname == '') return 0;
		
		$oldname = $this->currDir.$oldname;
		$newname = $this->currDir.$newname;
		
		if (file_exists($newname))
			return 0;
		
		if (file_exists($oldname) && file_ext($oldname) == file_ext($newname))
			return rename($oldname, $newname);
			
		return 0;
	}
	
	public function getMaxFileSize($measure = 'kb')
	{
		$filesize = $this->maxFileSize;
		if ($measure == 'kb')
			$filesize = floor($filesize / 1024);
		return $filesize;
	}
}